#include "mcc_generated_files/mcc.h"

// PI制御パラメータ
#define Kp 2   // 比例ゲイン
#define Ki 1  // 積分ゲイン

// PI制御用変数
int error = 0;       // 目標値と現在値の差
int integral = 0;    // 積分項
int pwm_output = 0;  // PWM出力

// 積分項の上下限値
#define INTEGRAL_MAX 1000
#define INTEGRAL_MIN -1000

// 上下限値設定関数
int constrain(int value, int min, int max) {
    if (value > max) {
        return max;
    } else if (value < min) {
        return min;
    }
    return value;
}

void TMR1_DefaultInterruptHandler(void){
    int v_wcom, v_w;
            
    // RC0ピンに1を出力
    RC0 = 1;

    //回転数検出値の読み込み
    v_w = (int)ADC_GetConversion(0x02);
    //回転数指令値の読み込み
    v_wcom = (int)ADC_GetConversion(0x06);        
            
 // 目標値と現在値の差を計算
    error = v_wcom - v_w; 
    
    // 積分項の計算
    integral += error;
    
    // 積分項の上下限値のチェック
    integral = constrain(integral, INTEGRAL_MIN, INTEGRAL_MAX);
    
    // PI制御アルゴリズム
    pwm_output = (Kp * error) + (Ki * integral);
    
    // PWM出力の制約条件
    pwm_output = constrain(pwm_output, 0, 999);
    
    // PWM出力の更新
    EPWM2_LoadDutyValue((unsigned int)pwm_output);
    
    // RC0ピンに0を出力
    RC0 = 0;
}
    
void main(void)
{
    // initialize the device
    SYSTEM_Initialize();

    while (1)
    { }
}

